@@ -13,7 +13,7 @@ module Variables = struct
 	(** Input: Variables (n,v) 
 		Output: Variables (m,v') with m>n and a new variable name x in v'  **)
 	let rec push (n,v) = 
-		let x = "x" ^ (string_of_int n) in
+		let x = "X" ^ (string_of_int n) in
 		let n = n+1 in
 		if (Hashtbl.mem coq_used_names x) 
 			then push (n,v)	    
@@ -283,6 +283,8 @@ with Not_found -> "missing_choice_axiom_for"^x
 
 let next_fresh_hyp : int ref = ref 0
 
+let next_fresh_trm : int ref = ref 0
+
 (** Input: unit
 	Output: returns next fresh hypothesis name **)
 let rec get_hyp_name hyp =
@@ -299,7 +301,6 @@ let rec find_fresh_consts n const =
 	let x =try Hashtbl.find coq_names x with
 	  Not_found
 	  ->
-	    Hashtbl.iter (fun x y -> Printf.printf "> %s %s\n" x y) coq_names; (* delete me *)
 	    failwith ("add_fresh_const can't find "^x^" in coq_names") in
 	if List.mem_assoc x const then [] else [(x,a)] 
     | Ap(m1,m2) -> find_fresh_consts m1 const @ find_fresh_consts m2 const
@@ -321,9 +322,9 @@ let rec lookup w s hyp =
     List.assoc s hyp
   with
   | Not_found ->
-      Printf.printf "%s: Could not find Coq hyp name\ns = %s\nhyp:\n" w (trm_str s);
+      Printf.printf "%s: Could not find hyp name\ns = %s\nhyp:\n" w (trm_str s);
       List.iter (fun (m,h) -> Printf.printf "%s: %s\n" h (trm_str m)) hyp;
-      failwith ("Could not find Coq hyp name")
+      failwith ("Could not find hyp name")
  
 (** Input: out_channel c, refutation r, association list (term -> hypothesis name) hyp, association list (constant name -> type) const, Space string sp 
 	Output: unit, prints refutation r to c **)
@@ -552,6 +553,593 @@ let ref_coq c r =
   Printf.fprintf c "Qed.\n";
   Printf.fprintf c "End SatallaxProblem.\n" 
 
+(*** July 2012 (Chad) : TSTP ***)
+(** Prints type m as a Tstp-formatted string on the out_channel c  **)
+let rec print_stp_tstp c m p =
+  match m with
+  | Base x ->
+      Printf.fprintf c "%s" x
+  | Prop ->
+      Printf.fprintf c "$o"
+  | Ar(a,b) ->
+      begin
+	if p then Printf.fprintf c "(";
+	print_stp_tstp c a true;
+	Printf.fprintf c ">";
+	print_stp_tstp c b false;
+	if p then Printf.fprintf c ")";
+	flush c
+      end
+
+let tstpizename x =
+  if (String.length x > 0) then
+    if (String.get x 0 = '_') then
+      "eigen" ^ x
+    else if (String.get x 0 = '@') then
+      String.sub x 1 ((String.length x) - 1)
+    else
+      if ((String.length x > 5) && (String.sub x 0 5 = "eigen")) then
+	"not" ^ x
+      else
+	x
+  else x
+
+(** Input: out_channel c, term m, list of bound variables 
+	Invariant: m is closed, if  it is enclosed in quantifiers for the bound variables 
+	Prints the term m on the channel c**)
+let rec trm_to_tstp c m bound =
+  match m with
+    Name(x,_) -> (* Definitions *)
+      Printf.fprintf c "%s" (tstpizename x)
+  | False -> (* Bottom *)
+	Printf.fprintf c "$false"
+  | Ap(Ap(Imp,m1),False) ->  (* Negation *)
+	begin
+	  Printf.fprintf c "(~(";
+	  trm_to_tstp c m1 bound;
+	  Printf.fprintf c "))";
+	end
+   | Ap(Ap(Imp,m1),m2) -> (* Implication *)
+	begin
+	  Printf.fprintf c "(";
+	  trm_to_tstp c m1 bound;
+	  Printf.fprintf c " => ";
+	  trm_to_tstp c m2 bound;
+	  Printf.fprintf c ")";
+	end
+  | Ap(Imp,m1) -> trm_to_tstp c (Lam(Prop,Ap(Ap(Imp,shift m1 0 1),DB(0,Prop)))) bound
+  | Imp -> trm_to_tstp c (Lam(Prop,Lam(Prop,Ap(Ap(Imp,DB(1,Prop)),DB(0,Prop))))) bound
+  | Ap(Forall(a),Lam(_,m1)) -> (* forall with Lam *)
+      begin
+	print_all_tstp c a m1 bound
+      end
+  | Forall(a) ->
+      begin
+	Printf.fprintf c "(!!)";
+      end
+  | Ap(Ap(Eq(a),m1),m2) -> (* Equality *)
+	begin
+	  Printf.fprintf c "(";
+	  trm_to_tstp c m1 bound;
+	  Printf.fprintf c " = ";
+	  trm_to_tstp c m2 bound;
+	  Printf.fprintf c ")"
+	end
+  | Eq(a) ->     
+      Printf.fprintf c "(=)"
+  | Ap(Choice(a),Lam(_,m1)) ->
+      let bound = Variables.push bound in
+      Printf.fprintf c "(@+["; Printf.fprintf c "%s" (Variables.top bound); 
+      Printf.fprintf c ":"; print_stp_tstp c a false; 
+      Printf.fprintf c "]:";
+      trm_to_tstp c m1 bound; Printf.fprintf c ")"
+  | Choice(a) ->
+      Printf.fprintf c "(@+)";
+  | True -> (* Top *)
+	Printf.fprintf c "$true"
+  | Ap(Ap(And,m1),m2) -> (* conjunction *)
+	begin
+	  Printf.fprintf c "(";
+	  trm_to_tstp c m1 bound;
+	  Printf.fprintf c " & ";
+	  trm_to_tstp c m2 bound;
+	  Printf.fprintf c ")";
+	end
+  | And ->Printf.fprintf c "(&)"
+  | Ap(Ap(Or,m1),m2) -> (* disjunction *)
+	begin
+	  Printf.fprintf c "(";
+	  trm_to_tstp c m1 bound;
+	  Printf.fprintf c " | ";
+	  trm_to_tstp c m2 bound;
+	  Printf.fprintf c ")";
+	end
+  | Or -> Printf.fprintf c "(|)"
+  | Ap(Ap(Iff,m1),m2) -> (* equivalenz *)
+	begin
+	  Printf.fprintf c "(";
+	  trm_to_tstp c m1 bound;
+	  Printf.fprintf c " <=> ";
+	  trm_to_tstp c m2 bound;
+	  Printf.fprintf c ")";
+	end
+  | Iff -> Printf.fprintf c "(<=>)"
+  | Neg -> Printf.fprintf c "(~)"
+  | Ap(Exists(a),Lam(_,m1)) -> (* exist *)
+      begin
+	print_ex_tstp c a m1 bound
+      end
+  | Exists(a) ->
+      begin
+	Printf.fprintf c "(??)";
+      end
+  | DB(i,a) -> (* Bound variable *)
+	Printf.fprintf c "%s" (Variables.get i bound)
+  | Lam(a,m) ->
+      begin
+	print_lam_tstp c a m bound
+      end
+  | Ap(m1,m2) ->     
+	begin
+	  Printf.fprintf c "(";
+	  trm_to_tstp c m1 bound;
+	  Printf.fprintf c " @ ";
+	  trm_to_tstp c m2 bound;
+	  Printf.fprintf c ")";
+	end      
+  | _ -> raise (GenericSyntaxError ("Unknown case in trm_to_tstp version : " ^ (trm_str m)))
+
+ (* Prints consecutive lambda-terms as a single fun in Tstp. *) 
+and print_lam_tstp c a m bound =
+	let bound = Variables.push bound in
+	Printf.fprintf c "(^["; Printf.fprintf c "%s" (Variables.top bound); Printf.fprintf c ":"; print_stp_tstp c a false; Printf.fprintf c "]:";
+	match m with
+		| Lam(b,m') -> print_lam_tstp c b m' bound; Printf.fprintf c ")"
+		| _ -> trm_to_tstp c m bound; Printf.fprintf c ")"
+
+(* Prints consecutive forall-terms together with the corresponding lambda-terms as a single forall in Tstp. *) 		
+and print_all_tstp c a m bound =
+  let bound = Variables.push bound in
+  Printf.fprintf c "(!["; Printf.fprintf c "%s" (Variables.top bound); Printf.fprintf c ":"; print_stp_tstp c a false; Printf.fprintf c "]:";
+  match m with
+  | Ap(Forall(a'),Lam(_,m')) -> print_all_tstp c a' m' bound; Printf.fprintf c ")"
+  | _ -> trm_to_tstp c m bound; Printf.fprintf c ")"
+
+(* Prints an exist-term together with the corresponding lambda-term as an exists in Tstp. *) 		
+and print_ex_tstp c a m bound =
+ 	let bound = Variables.push bound in
+	Printf.fprintf c "(?["; Printf.fprintf c "%s" (Variables.top bound); 
+	Printf.fprintf c ":"; print_stp_tstp c a false; 
+        Printf.fprintf c "]:";
+	trm_to_tstp c m bound; Printf.fprintf c ")"
+
+let tstp_axioms : (trm,string * trm * out_channel * (bool ref)) Hashtbl.t = Hashtbl.create 100;;
+
+let forallbvarnames : (string,string) Hashtbl.t = Hashtbl.create 100;;
+let nexistsbvarnames : (string,string) Hashtbl.t = Hashtbl.create 100;;
+
+let trm_to_tstp_rembvar x c m bound =
+  match m with
+  | Ap(Forall(a),Lam(_,m1)) ->
+      let bound = Variables.push bound in
+      let y = Variables.top bound in
+      Hashtbl.add forallbvarnames x y;
+      Printf.fprintf c "(!["; Printf.fprintf c "%s" y; Printf.fprintf c ":"; print_stp_tstp c a false; Printf.fprintf c "]:";
+      trm_to_tstp c m1 bound; Printf.fprintf c ")"
+  | Ap(Neg,Ap(Exists(a),Lam(_,m1))) ->
+      let bound = Variables.push bound in
+      let y = Variables.top bound in
+      Hashtbl.add nexistsbvarnames x y;
+      Printf.fprintf c "(~(?["; Printf.fprintf c "%s" y; Printf.fprintf c ":"; print_stp_tstp c a false; Printf.fprintf c "]:";
+      trm_to_tstp c m1 bound; Printf.fprintf c "))"
+  | Ap(Ap(Imp,Ap(Exists(a),Lam(_,m1))),False) ->
+      let bound = Variables.push bound in
+      let y = Variables.top bound in
+      Hashtbl.add nexistsbvarnames x y;
+      Printf.fprintf c "(~(?["; Printf.fprintf c "%s" y; Printf.fprintf c ":"; print_stp_tstp c a false; Printf.fprintf c "]:";
+      trm_to_tstp c m1 bound; Printf.fprintf c "))"
+  | _ ->
+      trm_to_tstp c m bound
+
+let rec lookup_tstp w s hyp =
+  try
+    List.assoc s hyp
+  with
+  | Not_found ->
+      begin
+	try
+	  let (x,m,c,f) = Hashtbl.find tstp_axioms s in
+	  if (!f) then
+	    begin
+	      f := false;
+	      Printf.fprintf c "thf(%s,axiom," x;
+	      trm_to_tstp_rembvar x c m (Variables.make ());
+	      Printf.fprintf c ").\n";
+	      flush c
+	    end;
+	  x
+	with
+	| Not_found ->
+	  Printf.printf "%s: Could not find hyp name\ns = %s\nhyp:\n" w (trm_str s);
+	  List.iter (fun (m,h) -> Printf.printf "%s: %s\n" h (trm_str m)) hyp;
+	  failwith ("Could not find hyp name")
+      end
+
+let tstpline : int ref = ref 1;;
+
+let tstphyp : (trm,string) Hashtbl.t = Hashtbl.create 100
+
+(**
+ Input: unit
+	Output: returns next fresh hypothesis name **)
+let rec get_thyp_name () =
+  let x = "h" ^ (string_of_int (!next_fresh_hyp)) in
+  incr next_fresh_hyp;
+  if (Hashtbl.mem coq_used_names x) 
+  then get_thyp_name ()
+  else
+    x
+
+let rec get_tstp_hyp_name c s =
+  try
+    Hashtbl.find tstphyp s
+  with Not_found ->
+    let x = "h" ^ (string_of_int (!next_fresh_hyp)) in
+    Hashtbl.add tstphyp s x;
+    incr next_fresh_hyp;
+    if (Hashtbl.mem coq_used_names x) 
+    then get_tstp_hyp_name c s
+    else
+      begin
+	Printf.fprintf c "thf(%s,assumption," x;
+	trm_to_tstp_rembvar x c s (Variables.make ());
+	Printf.fprintf c ",introduced(assumption,[])).\n";
+	flush c;
+	x
+      end
+
+let rec as_str2 al =
+  match al with
+  | (a::ar) -> "," ^ a ^ (as_str2 ar)
+  | [] -> ""
+
+let as_str al =
+  match al with
+  | (a::ar) -> "[" ^ a ^ (as_str2 ar) ^ "]"
+  | [] -> "[]"
+
+let rec disch_str2 d =
+  match d with
+  | (x::r) -> "," ^ x ^ (disch_str2 r)
+  | [] -> ""
+
+let disch_str d =
+  match d with
+  | (x::r) -> x ^ (disch_str2 r)
+  | [] -> ""
+
+let rec info_str2 r dl =
+  match dl with
+  | (d::dr) -> "," ^ r ^ "(discharge,[" ^ (disch_str d) ^ "])" ^ (info_str2 r dr)
+  | [] -> ""
+
+let info_str r hyp dl =
+  match hyp with
+  | [] -> info_str2 r dl
+  | _ -> ",assumptions(" ^ (as_str (List.map (fun (_,h) -> h) hyp)) ^ ")" ^ (info_str2 r dl)
+
+(** Input: out_channel c, refutation r, association list (term -> hypothesis name) hyp, association list (constant name -> type) const, Space string sp 
+	Output: unit, prints refutation r to c **)
+let rec ref_tstp1 c r hyp const =
+  let l = !tstpline in
+  incr tstpline;
+  begin
+    match r with
+    | Conflict(s,ns) ->
+	let p1 = (lookup_tstp "0" (coqnorm s) hyp) in
+	let p2 = (lookup_tstp "1" (coqnorm ns) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_conflict,[status(thm)%s],[%s,%s])).\n" l (info_str "tab_conflict" hyp []) p1 p2
+    | Fal(_) -> 				
+	let p1 = (lookup_tstp "2" False hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_false,[status(thm)%s],[%s])).\n" l (info_str "tab_false" hyp []) p1
+    | NegRefl(s) -> 			
+	let p1 = (lookup_tstp "3" (coqnorm s) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_refl,[status(thm)%s],[%s])).\n" l (info_str "tab_refl" hyp []) p1;
+    | Implication(h,s,t,r1,r2) -> 	
+	let h1 = get_tstp_hyp_name c (coqnorm s) in	
+	let h2 = get_tstp_hyp_name c (coqnorm t) in	
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::hyp) const in
+	let l2 = ref_tstp1 c r2 ((coqnorm t,h2)::hyp) const in
+	let p1 = (lookup_tstp "4" (coqnorm h) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_imp,[status(thm)%s],[%s,%d,%d%s])).\n" l (info_str "tab_imp" hyp [[h1];[h2]]) p1 l1 l2 (disch_str2 [h1;h2]);
+    | Disjunction(h,s,t,r1,r2) ->
+	let h1 = get_tstp_hyp_name c (coqnorm s) in	
+	let h2 = get_tstp_hyp_name c (coqnorm t) in	
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::hyp) const in
+	let l2 = ref_tstp1 c r2 ((coqnorm t,h2)::hyp) const in
+	let p1 = (lookup_tstp "5" (coqnorm h) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_or,[status(thm)%s],[%s,%d,%d%s])).\n" l (info_str "tab_or" hyp [[h1];[h2]]) p1 l1 l2 (disch_str2 [h1;h2]);
+    | NegConjunction(h,s,t,r1,r2) ->
+	let h1 = get_tstp_hyp_name c (coqnorm s) in	
+	let h2 = get_tstp_hyp_name c (coqnorm t) in	
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::hyp) const in
+	let l2 = ref_tstp1 c r2 ((coqnorm t,h2)::hyp) const in
+	let p1 = (lookup_tstp "6" (coqnorm h) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_nand,[status(thm)%s],[%s,%d,%d%s])).\n" l (info_str "tab_nand" hyp [[h1];[h2]]) p1 l1 l2 (disch_str2 [h1;h2]);
+    | NegImplication(h,s,t,r1) ->
+	let h1 = get_tstp_hyp_name c (coqnorm s) in
+	let h2 = get_tstp_hyp_name c (coqnorm t) in	
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::(coqnorm t,h2)::hyp) const in
+	let p1 = (lookup_tstp "7" (coqnorm h) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_negimp,[status(thm)%s],[%s,%d%s])).\n" l (info_str "tab_negimp" hyp [[h1;h2]]) p1 l1 (disch_str2 [h1;h2]);
+    | Conjunction(h,s,t,r1) ->
+	let h1 = get_tstp_hyp_name c (coqnorm s) in
+	let h2 = get_tstp_hyp_name c (coqnorm t) in	
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::(coqnorm t,h2)::hyp) const in
+	let p1 = (lookup_tstp "8" (coqnorm h) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_and,[status(thm)%s],[%s,%d%s])).\n" l (info_str "tab_and" hyp [[h1;h2]]) p1 l1 (disch_str2 [h1;h2]);
+    | NegDisjunction(h,s,t,r1) ->
+	let h1 = get_tstp_hyp_name c (coqnorm s) in
+	let h2 = get_tstp_hyp_name c (coqnorm t) in	
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::(coqnorm t,h2)::hyp) const in
+	let p1 = (lookup_tstp "9" (coqnorm h) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_nor,[status(thm)%s],[%s,%d%s])).\n" l (info_str "tab_nor" hyp [[h1;h2]]) p1 l1 (disch_str2 [h1;h2]);
+    | All(h,s,r1,a,m,n) ->
+	let const = add_fresh_const c const n "" in
+	let h1 = get_tstp_hyp_name c (coqnorm s) in
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::hyp) const in
+	let p1 = (lookup_tstp "9" (coqnorm h) hyp) in
+	begin
+	  try
+	    let x = Hashtbl.find forallbvarnames p1 in
+	    Printf.fprintf c "thf(%d,plain,$false,inference(tab_all,[status(thm)%s],[%s:[bind(%s,$thf(" l (info_str "tab_all" hyp [[h1]]) p1 x;
+	    trm_to_tstp c n (Variables.make ());
+	    Printf.fprintf c "))],%d%s])).\n" l1 (disch_str2 [h1])
+	  with Not_found ->
+	    Printf.fprintf c "thf(%d,plain,$false,inference(tab_all,[status(thm)%s],[%s:[bind(Xnoname,$thf(" l (info_str "tab_all" hyp [[h1]]) p1;
+	    trm_to_tstp c n (Variables.make ());
+	    Printf.fprintf c "))],%d%s])).\n" l1 (disch_str2 [h1])
+	end
+    | NegAll(h,s,r1,a,m,x) ->
+	let h1 = get_tstp_hyp_name c (coqnorm s) in
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::hyp) ((x,a)::const) in
+	let p1 = (lookup_tstp "11" (coqnorm h) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_negall,[status(thm)%s,tab_negall(eigenvar,%s)],[%s,%d%s])).\n" l (info_str "tab_negall" hyp [[h1]]) (tstpizename x) p1 l1 (disch_str2 [h1])
+    | Exist(h,s,r1,a,m,x) ->
+	let h1 = get_tstp_hyp_name c (coqnorm s) in
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::hyp) ((x,a)::const) in
+	let p1 = (lookup_tstp "11" (coqnorm h) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_ex,[status(thm)%s,tab_ex(eigenvar,%s)],[%s,%d%s])).\n" l (info_str "tab_ex" hyp [[h1]]) (tstpizename x) p1 l1 (disch_str2 [h1])
+    | NegExist(h,s,r1,a,m,n) ->
+	let const = add_fresh_const c const n "" in
+	let h1 = get_tstp_hyp_name c (coqnorm s) in	
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::hyp) const in
+	let p1 = (lookup_tstp "12" (coqnorm h) hyp) in
+	begin
+	  try
+	    let x = Hashtbl.find nexistsbvarnames p1 in
+	    Printf.fprintf c "thf(%d,plain,$false,inference(tab_negex,[status(thm)%s],[%s:[bind(%s,$thf(" l (info_str "tab_negex" hyp [[h1]]) p1 x;
+	    trm_to_tstp c n (Variables.make ());
+	    Printf.fprintf c "))],%d%s])).\n" l1 (disch_str2 [h1])
+	  with Not_found ->
+	    Printf.fprintf c "thf(%d,plain,$false,inference(tab_negex,[status(thm)%s],[%s:[bind(Xnoname,$thf(" l (info_str "tab_negex" hyp [[h1]]) p1;
+	    trm_to_tstp c n (Variables.make ());
+	    Printf.fprintf c "))],%d%s])).\n" l1 (disch_str2 [h1])
+	end
+    | Mating(h1,h2, ss, rs) ->
+	let hl = ref [] in
+	let ll = ref [] in
+	let fst = ref true in
+	List.iter (fun (s,r) ->
+	  let h' = get_tstp_hyp_name c (coqnorm s) in
+	  let l' = ref_tstp1 c r ((coqnorm s,h')::hyp) const in
+	  hl := h'::!hl;
+	  ll := l'::!ll)
+	  (List.combine ss rs);
+	let p1 = (lookup_tstp "14" (coqnorm h1) hyp) in
+	let p2 = (lookup_tstp "15" (coqnorm h2) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_mat,[status(thm)%s],[%s,%s" l (info_str "tab_mat" hyp (List.map (fun h -> [h]) (!hl))) p1 p2;
+	flush c;
+	Printf.fprintf c "],[%s,%s" p1 p2;
+	flush c;
+	List.iter (fun l' ->
+	  Printf.fprintf c ",%d" l'; flush c
+	      ) !ll;
+	Printf.fprintf c "%s])).\n" (disch_str2 !hl)
+    | Decomposition(h1, ss, rs) ->
+	let hl = ref [] in
+	let ll = ref [] in
+	let fst = ref true in
+	List.iter (fun (s,r) ->
+	  let h' = get_tstp_hyp_name c (coqnorm s) in
+	  let l' = ref_tstp1 c r ((coqnorm s,h')::hyp) const in
+	  hl := h'::!hl;
+	  ll := l'::!ll)
+	  (List.combine ss rs);
+	let p1 = (lookup_tstp "16" (coqnorm h1) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_dec,[status(thm)%s],[%s" l (info_str "tab_dec" hyp (List.map (fun h -> [h]) (!hl))) p1;
+	flush c;
+	List.iter (fun l' ->
+	    Printf.fprintf c ",%d" l'; flush c
+	      ) !ll;
+	Printf.fprintf c "%s])).\n" (disch_str2 !hl)
+    | Confront(h1,h2,su,tu,sv,tv,r1,r2) ->
+	let h3 = get_tstp_hyp_name c (coqnorm su) in
+	let h4 = get_tstp_hyp_name c (coqnorm tu) in
+	let h5 = get_tstp_hyp_name c (coqnorm sv) in
+	let h6 = get_tstp_hyp_name c (coqnorm tv) in
+	let l1 = ref_tstp1 c r1 ((coqnorm su,h3)::(coqnorm tu,h4)::hyp) const in
+	let l2 = ref_tstp1 c r2 ((coqnorm sv,h5)::(coqnorm tv,h6)::hyp) const in
+	let p1 = (lookup_tstp "17" (coqnorm h1) hyp) in
+	let p2 = (lookup_tstp "18" (coqnorm h2) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_con,[status(thm)%s],[%s,%s,%d,%d%s])).\n" l (info_str "tab_con" hyp [[h3;h4];[h5;h6]]) p1 p2 l1 l2 (disch_str2 [h3;h4;h5;h6])
+    | Trans(h1,h2,su,r1) ->
+	let h3 = get_tstp_hyp_name c (coqnorm su) in	
+	let l1 = ref_tstp1 c r1 ((coqnorm su,h3)::hyp) const in
+	let p1 = (lookup_tstp "17" (coqnorm h1) hyp) in
+	let p2 = (lookup_tstp "18" (coqnorm h2) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_trans,[status(thm)%s],[%s,%s,%d%s])).\n" l (info_str "tab_trans" hyp [[h3]]) p1 p2 l1 (disch_str2 [h3])
+    | NegEqualProp(h,s,t,r1,r2) -> 
+	let h1 = get_tstp_hyp_name c (coqnorm s) in
+	let h2 = get_tstp_hyp_name c (coqnorm (neg t)) in
+	let h3 = get_tstp_hyp_name c (coqnorm (neg s)) in
+	let h4 = get_tstp_hyp_name c (coqnorm t) in
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::(coqnorm (neg t),h2)::hyp) const in
+	let l2 = ref_tstp1 c r2 ((coqnorm (neg s),h3)::(coqnorm t,h4)::hyp) const in
+	let p1 = (lookup_tstp "21" (coqnorm h) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_be,[status(thm)%s],[%s,%d,%d%s])).\n" l (info_str "tab_be" hyp [[h1;h2];[h3;h4]]) p1 l1 l2 (disch_str2 [h1;h2;h3;h4])
+    | EqualProp(h,s,t,r1,r2) -> 
+	let h1 = get_tstp_hyp_name c (coqnorm s) in
+	let h2 = get_tstp_hyp_name c (coqnorm t) in	
+	let h3 = get_tstp_hyp_name c (coqnorm (neg s)) in
+	let h4 = get_tstp_hyp_name c (coqnorm (neg t)) in
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::(coqnorm t,h2)::hyp) const in
+	let l2 = ref_tstp1 c r2 ((coqnorm (neg s),h3)::(coqnorm (neg t),h4)::hyp) const in
+	let p1 = (lookup_tstp "22" (coqnorm h) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_bq,[status(thm)%s],[%s,%d,%d%s])).\n" l (info_str "tab_bq" hyp [[h1;h2];[h3;h4]]) p1 l1 l2 (disch_str2 [h1;h2;h3;h4])
+    | NegAequivalenz(h,s,t,r1,r2) -> 
+	let h1 = get_tstp_hyp_name c (coqnorm s) in
+	let h2 = get_tstp_hyp_name c (coqnorm (neg t)) in
+	let h3 = get_tstp_hyp_name c (coqnorm (neg s)) in
+	let h4 = get_tstp_hyp_name c (coqnorm t) in
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::(coqnorm (neg t),h2)::hyp) const in
+	let l2 = ref_tstp1 c r2 ((coqnorm (neg s),h3)::(coqnorm t,h4)::hyp) const in
+	let p1 = (lookup_tstp "23" (coqnorm h) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_negiff,[status(thm)%s],[%s,%d,%d%s])).\n" l (info_str "tab_negiff" hyp [[h1;h2];[h3;h4]]) p1 l1 l2 (disch_str2 [h1;h2;h3;h4])
+    | Aequivalenz(h,s,t,r1,r2) -> 
+	let h1 = get_tstp_hyp_name c (coqnorm s) in
+	let h2 = get_tstp_hyp_name c (coqnorm t) in	
+	let h3 = get_tstp_hyp_name c (coqnorm (neg s)) in
+	let h4 = get_tstp_hyp_name c (coqnorm (neg t)) in
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::(coqnorm t,h2)::hyp) const in
+	let l2 = ref_tstp1 c r2 ((coqnorm (neg s),h3)::(coqnorm (neg t),h4)::hyp) const in
+	let p1 = (lookup_tstp "24" (coqnorm h) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_iff,[status(thm)%s],[%s,%d,%d%s])).\n" l (info_str "tab_iff" hyp [[h1;h2];[h3;h4]]) p1 l1 l2 (disch_str2 [h1;h2;h3;h4])
+    | NegEqualFunc(h,s,r1) ->
+	let h1 = get_tstp_hyp_name c (coqnorm s) in	
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::hyp) const in
+	let p1 = (lookup_tstp "25" (coqnorm h) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_fe,[status(thm)%s],[%s,%d%s])).\n" l (info_str "tab_fe" hyp [[h1]]) p1 l1 (disch_str2 [h1])
+    | EqualFunc(h,s,r1) ->
+	let h1 = get_tstp_hyp_name c (coqnorm s) in	
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::hyp) const in
+	let p1 = (lookup_tstp "26" (coqnorm h) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_fq,[status(thm)%s],[%s,%d%s])).\n" l (info_str "tab_fq" hyp [[h1]]) p1 l1 (disch_str2 [h1])
+    | ChoiceR(eps,pred,s,t,r1,r2) -> 
+	let const = add_fresh_const c const pred "" in
+	let h1 = get_tstp_hyp_name c (coqnorm s) in
+	let h2 = get_tstp_hyp_name c (coqnorm t) in
+	begin
+	  match eps with
+	  | Choice(a) -> 
+	      let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::hyp) const in
+	      let l2 = ref_tstp1 c r2 ((coqnorm t,h2)::hyp) const in
+	      Printf.fprintf c "thf(%d,plain,$false,inference(tab_choice,[status(thm)%s],[%d,%d%s])).\n" l (info_str "tab_choice" hyp [[h1];[h2]]) l1 l2 (disch_str2 [h1;h2])
+	  | Name(x,Ar(Ar(a,Prop),_)) ->
+	      let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::hyp) const in
+	      let l2 = ref_tstp1 c r2 ((coqnorm t,h2)::hyp) const in
+	      Printf.fprintf c "thf(%d,plain,$false,inference(tab_choiceop,[status(thm),%s,%s%s],[%d,%d%s])).\n" l x (get_Choicop_axiom x a hyp) (info_str "tab_choiceop" hyp [[h1];[h2]]) l1 l2 (disch_str2 [h1;h2])
+	  | _ -> failwith "eps is not a valid epsilon"
+	end
+    | Cut(s,r1,r2) -> 
+	let const = add_fresh_const c const s "" in
+	let h1 = get_tstp_hyp_name c (coqnorm s) in
+	let h2 = get_tstp_hyp_name c (coqnorm (neg s)) in
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::hyp) const in
+	let l2 = ref_tstp1 c r2 ((coqnorm (neg s),h2)::hyp) const in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_cut,[status(thm)%s],[%d,%d%s])).\n" l (info_str "tab_cut" hyp [[h1];[h2]]) l1 l2 (disch_str2 [h1;h2])
+    | DoubleNegation(h,s,r1) ->
+	let h1 = get_tstp_hyp_name c (coqnorm s) in
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::hyp) const in
+	let p1 = (lookup_tstp "27" (coqnorm h) hyp) in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_dn,[status(thm)%s],[%s,%d%s])).\n" l (info_str "tab_dn" hyp [[h1]]) p1 l1 (disch_str2 [h1])
+    | Rewrite(prefix,pt,pt',r1) ->
+	let h =  coqnorm (Ap(prefix,pt)) in
+	let h1 = lookup_tstp "28" h hyp in	
+	let s =  coqnorm (Ap(prefix,pt')) in 
+	let h2 = get_tstp_hyp_name c s in
+	let l2 = ref_tstp1 c r1 ((s,h2)::hyp) const in
+	let rewout rname =
+	  Printf.fprintf c "thf(%d,plain,$false,inference(%s,[status(thm)%s,%s(leibnizp,$thf(" l rname (info_str rname hyp [[h2]]) rname;
+	  trm_to_tstp c prefix (Variables.make ());
+	  Printf.fprintf c "))],[%s,%d%s])).\n" h1 l2 (disch_str2 [h2])
+	in
+	begin
+	  match pt with
+	  | True -> rewout "tab_rew_true"
+	  | And -> rewout "tab_rew_and"
+	  | Or -> rewout "tab_rew_or"
+	  | Iff -> rewout "tab_rew_iff"
+	  | Exists(_) -> rewout "tab_rew_ex"
+	  | Eq(_) -> rewout "tab_rew_sym"
+	  | Lam(_,Lam(_,Ap(DB(1,_),DB(0,_)))) ->  rewout "tab_rew_eta"
+	  | Lam(Ar(Prop,Prop),Ap(Ap(Imp,Ap(Ap(Imp,DB(0,Prop)),False)),False)) ->  rewout "tab_rew_dn"
+	  | Lam(_,Lam(_,Ap(Forall(_),Lam(_,(Ap(Ap(Imp,(Ap(DB(0,_),DB(2,_)))),(Ap(DB(0,_),DB(1,_)))) ))) )) ->  rewout "tab_rew_leib1"
+	  | Lam(_,Lam(_,Ap(Forall(_),Lam(_,(Ap(Ap(Imp,Ap(Ap(Imp,(Ap(DB(0,_),DB(2,_)))),False)),Ap(Ap(Imp,(Ap(DB(0,_),DB(1,_)))),False)) ))) )) ->  rewout "tab_rew_leib2"
+	  | Lam(_,Lam(_,Ap(Forall(_),Lam(_,(Ap(Ap(Imp,(Ap(Forall(_),Lam(_,(Ap(Ap(DB(1,_),DB(0,_)),DB(0,_)))))) ),(Ap(Ap(DB(0,_),DB(2,_)),DB(1,_)))) ) )) )) ->  rewout "tab_rew_leib3"
+	  | Lam(_,Lam(_, Ap(Forall(_),Lam(_,(Ap(Ap(Imp,(Ap(Ap(Imp,(Ap(Ap(DB(0,_),DB(2,_)),DB(1,_)))),False) )),(Ap(Ap(Imp,(Ap(Forall(_),Lam(_,(Ap(Ap(DB(1,_),DB(0,_)),DB(0,_))))) )),False) )) ) )) )) ->  rewout "tab_rew_leib4"
+	  | _ -> failwith("unknown rewrite step found in ref_tstp" ^ (trm_str pt))
+	end
+    | Delta(h,s,x,r1) ->
+	let h1 = (lookup_tstp "29" (coqnorm h) hyp) in
+	let h2 = get_tstp_hyp_name c (coqnorm s) in
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h2)::hyp) const in
+	Printf.fprintf c "thf(%d,plain,$false,inference(tab_delta,[status(thm),%s%s],[%s,%d%s])).\n" l x (info_str "tab_delta" hyp [[h2]]) h1 l1 (disch_str2 [h2])
+    | KnownResult(s,name,al,r1) ->
+	let h1 = get_tstp_hyp_name c (coqnorm s) in
+	let l1 = ref_tstp1 c r1 ((coqnorm s,h1)::hyp) const in
+	begin
+	  Printf.fprintf c "thf(%d,plain,$false,inference(tab_known,[status(thm)%s" l (info_str "tab_known" hyp [[h1]]);
+	  flush c;
+	  List.iter (fun a ->
+	    Printf.fprintf c ",$thf(";
+	    flush c;
+	    print_stp_tstp c a false;
+	    Printf.fprintf c ")";
+	    flush c;
+	    )
+	    al;
+	  Printf.fprintf c "],[%s,%d%s])).\n" (tstpizename name) l1 (disch_str2 [h1])
+	end
+    | NYI(h,s,r1) -> failwith("NYI step found in ref_tstp" )
+    | Timeout -> failwith("Timeout step found in ref_tstp" )
+    | _ -> failwith("unknown refutation case in ref_tstp" )    
+  end; flush c; l
+
+ (** Prints refutation r to out_channel c **)
+let ref_tstp c r = 
+  List.iter
+    (fun (s,pt) ->
+      let m = coqnorm pt in
+      Hashtbl.add tstp_axioms m (s,m,c,ref true))
+    (!coqsig_hyp_trm);
+  match !conjecture with
+  | Some(_,con,_) ->
+    let ccon = coqnorm con in
+    let ncon = neg ccon in
+    let h1 = get_thyp_name() in
+    Printf.fprintf c "thf(%s,conjecture," (!conjecturename);
+    flush c;
+    trm_to_tstp c ccon (Variables.make ());
+    flush c;
+    Printf.fprintf c ").\n";
+    flush c;
+    Printf.fprintf c "thf(%s,negated_conjecture," h1;
+    flush c;
+    trm_to_tstp_rembvar h1 c ncon (Variables.make ());
+    flush c;
+    Printf.fprintf c ",inference(assume_negation,[status(cth)],[%s])).\n" (!conjecturename);
+    flush c;
+    let ll = ref_tstp1 c r [(ncon,h1)] (!coqsig_const) in
+    Printf.fprintf c "thf(0,theorem,";
+    flush c;
+    trm_to_tstp c ccon (Variables.make ());
+    flush c;
+    Printf.fprintf c ",inference(contra,[status(thm),contra(discharge,[%s])],[%d,%s])).\n" h1 ll h1;
+    flush c;
+  | None ->
+    ignore (ref_tstp1 c r [] (!coqsig_const))
+
 (*** Oct 2011 (Chad): A version for simply typed version in Coq. ***)
 
 (** Input: stp a ***)
@@ -744,7 +1332,7 @@ let rec coq_sterm c m bound lp rp =
 	  coq_sterm c m2 bound 5001 (-1);
 	  Printf.fprintf c ")";
 	end      
-  | _ -> raise (GenericSyntaxError ("Unknown case in trm_to_coq version : " ^ (trm_str m)))
+  | _ -> raise (GenericSyntaxError ("Unknown case in coq_sterm version : " ^ (trm_str m)))
 
  (* Prints consecutive lambda-terms as a single fun in Coq. *) 
 and coq_slam c a m bound =
