@@ -24,73 +24,65 @@ let counterstrategy_schedule_1 = [
 ("satmode1",10.0);
 ];;
 
-(*** Satallax 2.4 schedule. ***)
-(*** 61 modes (3 repeated), 260.8 total time ***)
+(*** Satallax 2.5 schedule. ***)
+(*** 56 modes, 226.1 total time, 1999 problems ***)
 let strategy_schedule_1 = [
-("mode205",0.1);
-("mode122c",0.1);
-("mode223m2",0.1);
-("mode222",0.2);
-("mode288",0.2);
+("mode175",0.1);
+("mode288",0.1);
+("mode238",0.1);
+("mode188",0.2);
+("mode19c",0.1);
 ("mode289",0.1);
-("mode245",0.1);
-("mode175",0.2);
-("mode290",0.3);
 ("mode252",0.5);
+("mode245",0.1);
+("mode249m2",0.1);
+("mode253",0.1);
+("mode293",0.5);
+("mode163",0.1);
+("mode236",0.2);
 ("mode295",0.2);
-("mode236",0.3);
-("mode284",0.3);
-("mode293",0.6);
-("mode257",0.3);
-("mode185",0.1);
 ("mode272",0.1);
-("mode233",1.0);
-("mode19c",0.2);
-("mode261",0.7);
-("mode213",1.3);
 ("mode244m5",0.4);
 ("mode4a",0.1);
-("mode238",0.1);
-("mode260",0.2);
-("mode205",1.0);
-("mode188",1.2);
-("mode299",0.3);
-("mode93",0.2);
-("mode86",0.8);
-("mode296",0.2);
-("mode204",8.1);
-("mode223m2",2.9);
-("mode267",1.0);
-("mode285",0.9);
-("mode297",0.3);
-("mode249m1",0.3);
-("mode294",0.4);
-("mode276",2.6);
-("mode298",1.1);
-("mode264",3.1);
-("mode286",5.7);
-("mode279",5.2);
-("mode122c",27.3);
-("mode251",4.1);
-("mode250",22.7);
-("mode273",4.3);
-("mode14",6.7);
-("mode283",13.5);
-("mode108",2.5);
-("mode6",2.7);
-("mode282",2.8);
-("mode173",12.8);
-("mode208",3.6);
-("mode170",3.8);
-("mode177",12.6);
-("mode280",4.4);
-("mode223",22.6);
-("mode274",10.9);
-("mode269",17.2);
-("mode252c",11.7);
-("mode165",20.5);
+("mode299",0.5);
+("mode233",0.8);
+("mode261",0.6);
+("mode290",0.5);
+("mode213",1.1);
+("mode185",0.1);
+("mode257",0.3);
+("mode93",0.1);
+("mode223m2",2.4);
+("mode204",6.6);
+("mode205",1.1);
+("mode86",0.7);
+("mode8a",0.4);
+("mode294",0.3);
+("mode276",2.3);
+("mode267",0.9);
+("mode264",2.4);
+("mode279",4.6);
+("mode251",3.2);
+("mode122c",21.1);
+("mode273",3.4);
+("mode172",3.0);
+("mode14",5.4);
+("mode250",19.2);
+("mode108",2.0);
+("mode282",2.1);
+("mode6",2.2);
+("mode223",17.4);
+("mode283",11.4);
+("mode208",3.1);
+("mode173",10.2);
+("mode280",3.9);
+("mode274",9.9);
+("mode269",14.0);
+("mode252c",9.0);
+("mode177",27.3);
+("mode165",18.5);
 ("mode301",1.0);
-("mode300",10.0);
+("mode300",10.0)
 ];;
 
 let print_help () =
@@ -118,7 +110,7 @@ let print_help () =
    print_newline();
    print_string "-G : A Coq file containing multiple conjectures is given. Try to prove each of them independently.";
    print_newline();
-   print_string "-p [coqscript|coqspfterm|hocore|modeltrue|model]: Output a kind of proof object";
+   print_string "-p [tstp|coqscript|coqspfterm|hocore|modeltrue|model]: Output a kind of proof object";
    print_newline());;
 
 (*** Load the mode (error if cannot find it) ***)
@@ -278,7 +270,9 @@ let process_command_line_args () =
 	    incr i;
 	    let pfkind = Sys.argv.(!i) in
 	    slaveargs := (pfkind::"-p"::!slaveargs);
-	    if ((String.lowercase pfkind) = "coqscript") then
+	    if ((String.lowercase pfkind) = "tstp") then
+	      mkproofterm := Some TSTP
+	    else if ((String.lowercase pfkind) = "coqscript") then
 	      mkproofterm := Some CoqScript
 	    else if ((String.lowercase pfkind) = "coqspfterm") then
 	      mkproofterm := Some CoqSPfTerm
@@ -331,6 +325,14 @@ let search_main () =
 	  begin
 	    basetypestobool := get_bool_flag "BASETYPESTOPROP";
 	    match (!mkproofterm) with
+	    | Some TSTP ->
+		begin
+		  if (not (!coq)) then
+		    begin (*** -p tstp implies -c - if -c was not given. Output TSTP proof via Coq out channel. - Chad, July 2012 ***)
+		      coq := true;
+		      coqoutchannel := stdout;
+		    end;
+		end
 	    | Some CoqScript ->
 		begin
 		  if (not (!coq)) then
@@ -373,6 +375,26 @@ let search_main () =
 	    begin
 	      begin
 		match (!mkproofterm,r) with
+		| (Some TSTP,Some r) ->
+		    if (!coqoutchannel = stdout) then
+		      begin
+			print_string "% SZS output start Proof"; print_newline();
+		      end;
+		    let c = !coqoutchannel in
+		    begin
+		      try print_tstp c r
+		      with CoqProofTooBig coqproofsize ->
+			begin
+			  if (!verbosity > 0) then Printf.printf "%% SZS status Success\nCoq Proof Too Big: %d steps\n" coqproofsize;
+			  exit 26;
+			end
+		    end;
+		    if (c = stdout) then
+		      begin
+			print_string "% SZS output end Proof"; print_newline();
+		      end
+		    else
+		      close_out c;
 		| (Some CoqScript,Some r) ->
 		    if (!coqoutchannel = stdout) then
 		      begin
