@@ -36,7 +36,7 @@ let rec updatecoqglobalsectionstack cx cgss co =
 
 let conjecturename : string ref = ref "claim"
 let conjecture : (pretrm * trm * trm) option ref = ref None
-type proofkind = CoqScript | CoqSPfTerm | HOCore | Model | ModelTrue
+type proofkind = TSTP | CoqScript | CoqSPfTerm | HOCore | Model | ModelTrue
 let mkproofterm = ref None
 let mkprooftermp () = match !mkproofterm with Some _ -> true | None -> false
 let slave1 = ref false
