@@ -30,7 +30,7 @@ val updatecoqglobalsectionstack : prectx -> (string * (out_channel -> unit) * pr
 
 val conjecturename : string ref
 val conjecture : (pretrm * trm * trm) option ref
-type proofkind = CoqScript | CoqSPfTerm | HOCore | Model | ModelTrue
+type proofkind = TSTP | CoqScript | CoqSPfTerm | HOCore | Model | ModelTrue
 val mkproofterm : proofkind option ref
 val mkprooftermp : unit -> bool
 val slave1 : bool ref
